@@ -199,9 +199,9 @@ bool ScimBridgeAgentAcceptListenerImpl::handle_event (scim_bridge_agent_event_ty
         //agent->quit ();
     } else {
         struct sockaddr_un empty_socket_addr;
-        socklen_t emtpy_socket_size = sizeof (empty_socket_addr);
+        socklen_t empty_socket_size = sizeof (empty_socket_addr);
 
-        int fd = accept (socket_fd, (sockaddr*) (&emtpy_socket_size), &emtpy_socket_size);
+        int fd = accept (socket_fd, (sockaddr*) (&empty_socket_addr), &empty_socket_size);
         if (fd < 0) {
             if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR) {
                 scim_bridge_perrorln ("Opps, it seems like failed to establish a connection with a client...");
