@@ -45,10 +45,6 @@ scim_bridge_debug_level_t scim_bridge_debug_get_level ()
 
 int main (int argc, char *argv[])
 {
-    setlocale(LC_ALL, "");
-    bindtextdomain (PACKAGE, SCIM_LOCALEDIR);
-    textdomain (PACKAGE);
-
     bool standalone_enabled = false;
     bool noexit_enabled = false;
 
