@@ -146,6 +146,7 @@ class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
         IMEngineInstancePointer imengine;
 
         bool enabled;
+        bool focused;
 
         scim_bridge_preedit_mode_t preedit_mode;
         bool preedit_shown;
@@ -274,7 +275,7 @@ void ScimBridgeAgentIMContext::set_on_the_spot_enabled (bool enabled)
 
 
 ScimBridgeAgentIMContextImpl::ScimBridgeAgentIMContextImpl (ScimBridgeAgentClientListener *new_client_listener):
-client_listener (new_client_listener), imengine (NULL), enabled (false), preedit_mode (PREEDIT_ANY),
+client_listener (new_client_listener), imengine (NULL), enabled (false), focused(false) ,preedit_mode (PREEDIT_ANY),
 preedit_shown (false), preedit_cursor_position (0)
 {
     if (free_imcontexts.empty ()) {
@@ -457,7 +458,7 @@ void ScimBridgeAgentIMContextImpl::set_enabled (bool new_state)
 
         hide_preedit ();
 
-        PanelFactoryInfo info = PanelFactoryInfo (String (""), String (_ ("English/Keyboard")), String ("C"), String (SCIM_KEYBOARD_ICON_FILE));
+        PanelFactoryInfo info = PanelFactoryInfo (String (""), String ("English/Keyboard"), String ("C"), String (SCIM_KEYBOARD_ICON_FILE));
         panel_listener->update_factory_info (info);
 
         panel_listener->turn_off ();
@@ -630,7 +631,7 @@ void ScimBridgeAgentIMContextImpl::focus_in ()
         if (factory.null ()) factory = fallback_imengine_factory;
         info = PanelFactoryInfo (factory->get_uuid (), utf8_wcstombs (factory->get_name ()), factory->get_language (), factory->get_icon_file ());
     } else {
-        info = PanelFactoryInfo (String (""), String (_ ("English/Keyboard")), String ("C"), String (SCIM_KEYBOARD_ICON_FILE));
+        info = PanelFactoryInfo (String (""), String ("English/Keyboard"), String ("C"), String (SCIM_KEYBOARD_ICON_FILE));
     }
     panel_listener->update_factory_info (info);
 
@@ -646,6 +647,8 @@ void ScimBridgeAgentIMContextImpl::focus_in ()
     panel_listener->hide_aux_string ();
     panel_listener->hide_lookup_table ();
     hide_preedit ();
+
+    focused = true;
 }
 
 
@@ -653,6 +656,9 @@ void ScimBridgeAgentIMContextImpl::focus_out ()
 {
     ScimBridgeAgentIMContext *focused_imcontext = static_cast<ScimBridgeAgentIMContext*> (get_imengine ()->get_frontend_data ());
 
+	if ( !focused )
+         return;
+
     get_imengine ()->set_frontend_data (static_cast<ScimBridgeAgentIMContext*> (this));
 
     hide_preedit ();
@@ -666,6 +672,8 @@ void ScimBridgeAgentIMContextImpl::focus_out ()
     } else {
         imengine->set_frontend_data (NULL);
     }
+
+    focused = false;
 }
 
 
@@ -915,20 +923,20 @@ void ScimBridgeAgentIMContextImpl::panel_commit_string (const WideString &wstr)
 
 void ScimBridgeAgentIMContextImpl::panel_request_help ()
 {
-    String help =  String (_ ("Smart Common Input Method platform ")) +
+    String help =  String ("Smart Common Input Method platform ") +
         String (SCIM_VERSION) +
-        String (_ ("\n(C) 2002-2005 James Su <suzhe@tsinghua.org.cn>\n\n"));
+        String ("\n(C) 2002-2005 James Su <suzhe@tsinghua.org.cn>\n\n");
 
         IMEngineFactoryPointer factory = scim_backend->get_factory (get_imengine ()->get_factory_uuid ());
         if (factory.null ()) factory = fallback_imengine_factory;
             help += utf8_wcstombs (factory->get_name ());
-            help += String (_ (":\n\n"));
+            help += String (":\n\n");
 
             help += utf8_wcstombs (factory->get_authors ());
-            help += String (_ ("\n\n"));
+            help += String ("\n\n");
 
             help += utf8_wcstombs (factory->get_help ());
-            help += String (_ ("\n\n"));
+            help += String ("\n\n");
 
             help += utf8_wcstombs (factory->get_credits ());
 
