@@ -15,7 +15,7 @@
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- */
+ */;
 
 #include <assert.h>
 #include <errno.h>
@@ -144,6 +144,7 @@ class ScimBridgeAgentImpl: public ScimBridgeAgent, public ScimBridgeAgentProtect
         ScimBridgeAgentSignalListener *signal_listener;
 
         ScimBridgeDisplay *display;
+        int valid_key_mask;
 
         retval_t initialize_scim ();
         retval_t finalize_scim ();
@@ -167,7 +168,7 @@ static bool is_socket_frontend_ready ()
         scim_bridge_pdebugln (8, "There is no socket frontend at all.");        
         return false;
     } else {
-        uint32 magic;
+        uint32_t magic;
         if (!scim_socket_open_connection (magic, "ConnectionTester", "SocketFrontEnd", socket_client, 1000)) {
             scim_bridge_pdebugln (8, "Failed to connect the socket fronent.");
             return false;
@@ -195,9 +196,6 @@ ScimBridgeAgentImpl::ScimBridgeAgentImpl ():
 running (true), noexit_enabled (false), standalone_enabled (false), client_app_count (0),
 accept_listener (NULL), interruption_listener (NULL), panel_listener (NULL), signal_listener (NULL), display (NULL)
 {
-    setlocale (LC_ALL, "");
-    bindtextdomain (GETTEXT_PACKAGE, SCIM_LOCALEDIR);
-    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 }
 
 
@@ -442,6 +440,9 @@ retval_t ScimBridgeAgentImpl::initialize_scim ()
         return RETVAL_FAILED;
     }
 
+	imengine_module_names.clear();
+	imengine_module_names.push_back("socket");
+
     // Get config modules
     vector<String> config_module_names;
     scim_get_config_module_list (config_module_names);
@@ -451,17 +452,11 @@ retval_t ScimBridgeAgentImpl::initialize_scim ()
         return RETVAL_FAILED;
     }
     
-    {
-    	scim_bridge_pdebugln (8, "Launching a SCIM daemon with x11 FrontEnd...");
-    	char *new_argv [] = {"--no-stay","-d", NULL};
-    	scim_launch (true, "socket", "socket", "x11", new_argv);
-    }
-
     // If there is no socket frontend running, launch one.
     if (!is_socket_frontend_ready ()) {
         scim_bridge_pdebugln (8, "Launching a SCIM daemon with Socket FrontEnd...");
         const String server_config_module_name = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_CONFIG_MODULE, String ("simple"));
-        char *new_argv [] = {"--no-stay","-d", NULL};
+        char* new_argv [] = {const_cast<char*>("--no-stay"),const_cast<char*>("-d"), NULL};
         scim_launch (true, server_config_module_name.c_str (), "all", "socket", new_argv);
 
         // Wait until the connection is established.
@@ -477,6 +472,12 @@ retval_t ScimBridgeAgentImpl::initialize_scim ()
         }
     }
     
+    {
+    	scim_bridge_pdebugln (8, "Launching a SCIM daemon with x11 FrontEnd...");
+    	char* new_argv [] = {const_cast<char*>("--no-stay"),const_cast<char*>("-d"), NULL};
+    	scim_launch (true, "socket", "socket", "x11", new_argv);
+    }
+
     // load config module
     scim_bridge_pdebugln (2, "Loading Config module...: socket");
     scim_config_module = new ConfigModule ("socket");
@@ -519,6 +520,18 @@ void ScimBridgeAgentImpl::slot_reload_config (const ConfigPointer &config)
 
     scim_frontend_hotkey_matcher.load_hotkeys (scim_config);
     scim_imengine_hotkey_matcher.load_hotkeys (scim_config);
+    
+    KeyEvent key;
+    scim_string_to_key (key,
+        config->read (String (SCIM_CONFIG_HOTKEYS_FRONTEND_VALID_KEY_MASK),
+                      String ("Shift+Control+Alt+Lock")));
+
+    int key_mask = (key.mask > 0) ? key.mask : 0xFFFF;
+    key_mask |= SCIM_KEY_ReleaseMask;
+    // Special treatment for two backslash keys on jp106 keyboard.
+    key_mask |= SCIM_KEY_QuirkKanaRoMask;
+    
+    valid_key_mask = key_mask;
 
     ScimBridgeAgentIMContext::set_enabled_by_default (scim_config->read (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), ScimBridgeAgentIMContext::is_enabled_by_default ()));
     ScimBridgeAgentIMContext::set_imengine_shared (scim_config->read (String (SCIM_CONFIG_FRONTEND_SHARED_INPUT_METHOD), ScimBridgeAgentIMContext::is_imengine_shared ()));
@@ -677,6 +690,7 @@ bool ScimBridgeAgentImpl::filter_key_event (scim_bridge_imcontext_id_t imcontext
     scim_bridge_pdebugln (5, "filter_key_event ()");
     
     KeyEvent new_key_event = key_event;
+    new_key_event.mask &= valid_key_mask;
     new_key_event.layout = scim_keyboard_layout;
 
     ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
