@@ -17,15 +17,17 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#include <alloca.h>
 #include <assert.h>
 #include <errno.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/un.h>
+#include <sys/wait.h>
 
 #include "scim-bridge-client.h"
 #include "scim-bridge-client-imcontext.h"
@@ -84,16 +86,32 @@ static IMContextList imcontext_list;
 
 static boolean initialized = FALSE;
 
-
 /* Helper Functions */
+static boolean check_scim_binary ()
+{
+    scim_bridge_pdebugln (1, "Checking SCIM binary...");
+    
+    FILE *pout = popen ("scim -h", "r");
+    if (pout != NULL) {
+        pclose (pout);
+        return TRUE;
+    } else {
+        scim_bridge_perrorln("Error (%d): %s", errno, strerror (errno));
+        return FALSE;
+    }
+}
+
 static retval_t launch_agent ()
 {
     scim_bridge_pdebugln (1, "Invoking the agent...");
-    if (system (scim_bridge_path_get_agent ()) != 0) {
+    
+    FILE *pout = popen (scim_bridge_path_get_agent (), "r");
+    if (pout != NULL) {
+        pclose (pout);
+        return RETVAL_SUCCEEDED;
+    } else {
         scim_bridge_perrorln ("Failed to invoking the agent: %s", strerror (errno));
         return RETVAL_FAILED;
-    } else {
-        return RETVAL_SUCCEEDED;
     }
 }
 
@@ -1015,6 +1033,11 @@ retval_t scim_bridge_client_open_messenger ()
         scim_bridge_perrorln ("The messenger has already been opend");
         return RETVAL_SUCCEEDED;
     }
+    
+    if (check_scim_binary () != TRUE) {
+        scim_bridge_perrorln ("There is no SCIM binary");
+        return RETVAL_FAILED;
+    }
 
     int i;
     for (i = 0; i < 10; ++i) {
