@@ -17,10 +17,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#include <alloca.h>
 #include <assert.h>
 #include <errno.h>
-#include <malloc.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -247,7 +245,7 @@ static boolean is_key_snooper_enabled ()
 static boolean is_precise_cursor_enabled ()
 {
     static boolean first_time = TRUE;
-    static boolean precise_cursor_enabled = TRUE;
+    static boolean precise_cursor_enabled = FALSE;
 
     if (first_time) {
         char *env_precise_cursor_enabled = getenv ("SCIM_BRIDGE_PRECISE_CURSOR_ENABLED");
@@ -872,7 +870,7 @@ void scim_bridge_client_imcontext_focus_in (GtkIMContext *context)
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
     
-    if (focused_imcontext != NULL) scim_bridge_client_imcontext_focus_out (GTK_IM_CONTEXT (focused_imcontext));
+    if (focused_imcontext != NULL && focused_imcontext != imcontext) scim_bridge_client_imcontext_focus_out (GTK_IM_CONTEXT (focused_imcontext));
     focused_imcontext = imcontext;
 
     if (!scim_bridge_client_is_messenger_opened () && scim_bridge_client_is_reconnection_enabled ()) {
