@@ -17,6 +17,20 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
+#include <cassert>
+#include <string>
+
+#ifdef QT4
+#include <QColor>
+#include <QInputMethodEvent>
+#include <QPalette>
+#include <QTextCharFormat>
+
+#ifdef Q_WS_X11
+#include <QX11Info>
+#endif
+#endif
+
 #include "scim-bridge-output.h"
 #include "scim-bridge-string.h"
 
@@ -25,6 +39,13 @@
 #include "scim-bridge-client-imcontext-qt.h"
 #include "scim-bridge-client-key-event-utility-qt.h"
 
+#ifdef QT4
+using namespace std;
+
+using namespace Qt;
+
+typedef QInputMethodEvent::Attribute QAttribute;
+#endif
 
 /* Static variables */
 class ScimBridgeClientIMContextImpl;
@@ -33,7 +54,6 @@ static ScimBridgeClientIMContextImpl *focused_imcontext = NULL;
 
 static bool key_event_forwarded = false;
 
-
 /* Class Definition */
 class ScimBridgeClientIMContextImpl: public _ScimBridgeClientIMContext
 {
@@ -45,13 +65,25 @@ class ScimBridgeClientIMContextImpl: public _ScimBridgeClientIMContext
 
         bool x11FilterEvent (QWidget *widget, XEvent *event);
         bool filterEvent (const QEvent *event);
-        void mouseHandler (int offset, QMouseEvent *event);
 
+#ifdef QT4
+        void update ();
+        QString identifierName ();
+        QString language ();
+
+        void setFocusWidget (QWidget *widget);
+        void widgetDestroyed (QWidget *widget);
+
+        bool isComposing () const;
+        void mouseHandler (int offset, QMouseEvent *event);
+#else
         void setFocus ();
         void unsetFocus ();
         void setMicroFocus (int x, int y, int w, int h, QFont *font = 0);
-        void reset ();
+        void mouseHandler (int offset, QEvent::Type type, ButtonState button, ButtonState state);
+#endif
 
+        void reset ();
 
         /* Semi private functions */
         void focus_in ();
@@ -71,6 +103,12 @@ class ScimBridgeClientIMContextImpl: public _ScimBridgeClientIMContext
         scim_bridge_imcontext_id_t get_id () const;
         void set_id (scim_bridge_imcontext_id_t new_id);
 
+#ifdef QT4
+        bool get_surrounding_text (unsigned int before_max, unsigned int after_max, char **string, int *cursor_position);
+        bool delete_surrounding_text (int offset, int length);
+        bool replace_surrounding_text (const char *text, int cursor_position);
+#endif
+
     private:
 
         scim_bridge_imcontext_id_t id;
@@ -79,11 +117,14 @@ class ScimBridgeClientIMContextImpl: public _ScimBridgeClientIMContext
 
         QString preedit_string;
 
-        int preedit_cursor_position;
-
+#ifdef QT4
+        QList<QAttribute> preedit_attributes;
+#else
         int preedit_selected_offset;
-
         int preedit_selected_length;
+#endif
+
+        int preedit_cursor_position;
 
         QString commit_string;
 
@@ -120,10 +161,17 @@ _ScimBridgeClientIMContext *_ScimBridgeClientIMContext::alloc ()
 }
 
 
-ScimBridgeClientIMContextImpl::ScimBridgeClientIMContextImpl (): id (-1), preedit_shown (false), preedit_selected_offset (0), preedit_selected_length (0)
+ScimBridgeClientIMContextImpl::ScimBridgeClientIMContextImpl (): id (-1), preedit_shown (false)
 {
     scim_bridge_pdebugln (5, "ScimBridgeClientIMContextImpl::ScimBridgeClientIMContextImpl ()");
-  
+
+#ifdef QT4
+    preedit_attributes.push_back (QAttribute (QInputMethodEvent::Cursor, preedit_cursor_position, true, 0));
+#else
+    preedit_selected_offset = 0;
+    preedit_selected_length = 0;
+#endif
+
     if (!scim_bridge_client_is_messenger_opened ()) {
         scim_bridge_perrorln ("The messenger is now down");
     } else if (scim_bridge_client_register_imcontext (this)) {
@@ -149,6 +197,81 @@ ScimBridgeClientIMContextImpl::~ScimBridgeClientIMContextImpl ()
     }
 }
 
+#ifdef QT4
+
+QString ScimBridgeClientIMContextImpl::identifierName ()
+{
+    return SCIM_BRIDGE_IDENTIFIER_NAME;
+}
+
+QString ScimBridgeClientIMContextImpl::language ()
+{
+    return "";
+}
+
+void ScimBridgeClientIMContextImpl::widgetDestroyed (QWidget *widget)
+{
+    scim_bridge_pdebugln (4, "ScimBridgeClientIMContextImpl::widgetDestroyed ()");
+    focus_out ();
+    update ();
+}
+
+void ScimBridgeClientIMContextImpl::setFocusWidget (QWidget *widget)
+{
+    scim_bridge_pdebugln (4, "ScimBridgeClientIMContextImpl::setFocusWidget ()");
+    QInputContext::setFocusWidget (widget);
+    focus_in ();
+    update ();
+}
+
+void ScimBridgeClientIMContextImpl::update ()
+{
+    scim_bridge_pdebugln (4, "ScimBridgeClientIMContextImpl::update ()");
+
+    QWidget *focused_widget = qApp->focusWidget ();
+    if (focused_widget != NULL) {
+        if (focused_imcontext == NULL)
+            focus_in ();
+
+        QRect rect = focused_widget->inputMethodQuery (ImMicroFocus).toRect ();
+        QPoint point (rect.x (), rect.y () + rect.height ());
+        QPoint new_cursor_location = focused_widget->mapToGlobal (point);
+        set_cursor_location (new_cursor_location);
+    }
+}
+
+bool ScimBridgeClientIMContextImpl::isComposing () const
+{
+    scim_bridge_pdebugln (4, "ScimBridgeClientIMContextImpl::isComposing ()");
+    return preedit_string.size () > 0;
+}
+
+void ScimBridgeClientIMContextImpl::mouseHandler (int offset, QMouseEvent *mevent)
+{
+}
+
+bool ScimBridgeClientIMContextImpl::get_surrounding_text (unsigned int before_max, unsigned int after_max, char **text, int *cursor_position)
+{
+    scim_bridge_pdebugln (6, "ScimBridgeClientIMContextImpl::get_surrounding_text ()");
+    scim_bridge_perrorln ("FIXME: scim_bridge_client_imcontext_delete_surrounding_text () is not yet implemented.");
+    return false;
+}
+
+bool ScimBridgeClientIMContextImpl::delete_surrounding_text (int offset, int length)
+{
+    scim_bridge_pdebugln (6, "ScimBridgeClientIMContextImpl::delete_surrounding_text ()");
+    scim_bridge_perrorln ("FIXME: scim_bridge_client_imcontext_delete_surrounding_text () is not yet implemented.");
+    return false;
+}
+
+bool ScimBridgeClientIMContextImpl::replace_surrounding_text (const char *text, int cursor_position)
+{
+    scim_bridge_pdebugln (6, "ScimBridgeClientIMContextImpl::replace_surrounding_text ()");
+    scim_bridge_perrorln ("FIXME: scim_bridge_client_imcontext_delete_surrounding_text () is not yet implemented.");
+    return false;
+}
+
+#else
 
 void ScimBridgeClientIMContextImpl::setFocus ()
 {
@@ -166,7 +289,7 @@ void ScimBridgeClientIMContextImpl::unsetFocus ()
 }
 
 
-void ScimBridgeClientIMContextImpl::setMicroFocus (int x, int y, int w, int h, QFont *font)
+void ScimBridgeClientIMContextImpl::setMicroFocus (int x, int y, int w, int h, QFont *qfont)
 {
     scim_bridge_pdebugln (4, "ScimBridgeClientIMContextImpl::setMicroFocus ()");
 
@@ -174,23 +297,23 @@ void ScimBridgeClientIMContextImpl::setMicroFocus (int x, int y, int w, int h, Q
     set_cursor_location (new_cursor_location);
 }
 
-
-void ScimBridgeClientIMContextImpl::mouseHandler (int offset, QMouseEvent *event)
+void ScimBridgeClientIMContextImpl::mouseHandler (int offset, QEvent::Type type, ButtonState button, ButtonState state)
 {
 }
+#endif
 
 
-bool ScimBridgeClientIMContextImpl::x11FilterEvent (QWidget *widget, XEvent *event)
+bool ScimBridgeClientIMContextImpl::x11FilterEvent (QWidget *widget, XEvent *xevent)
 {
     scim_bridge_pdebugln (5, "ScimBridgeClientIMContextImpl::x11FilterEvent ()");
 
 #ifdef Q_WS_X11
-    if (key_event_forwarded || (event->type != XKeyPress && event->type != XKeyRelease)) return false;
+    if (key_event_forwarded || (xevent->type != XKeyPress && xevent->type != XKeyRelease)) return false;
     
     if (focused_imcontext != this) focus_in ();
 
     if (scim_bridge_client_is_messenger_opened ()) {
-        ScimBridgeKeyEvent *bridge_key_event = scim_bridge_key_event_x11_to_bridge (event);
+        ScimBridgeKeyEvent *bridge_key_event = scim_bridge_key_event_x11_to_bridge (xevent);
 
         boolean consumed = FALSE;
         const retval_t retval_error = scim_bridge_client_handle_key_event (this, bridge_key_event, &consumed);
@@ -209,17 +332,17 @@ bool ScimBridgeClientIMContextImpl::x11FilterEvent (QWidget *widget, XEvent *eve
 }
 
 
-bool ScimBridgeClientIMContextImpl::filterEvent (const QEvent *event)
+bool ScimBridgeClientIMContextImpl::filterEvent (const QEvent *qevent)
 {
     scim_bridge_pdebugln (5, "ScimBridgeClientIMContextImpl::filterEvent ()");
 
 #ifndef Q_WS_X11
-    if (key_event_forwarded || (event->type () != QEvent::KeyPress && event->type () != QEvent::KeyRelease)) return false;
+    if (key_event_forwarded || (qevent->type () != QEvent::KeyPress && qevent->type () != QEvent::KeyRelease)) return false;
     
     if (focused_imcontext != this) focus_in ();
 
     if (scim_bridge_client_is_messenger_opened ()) {
-        const QKeyEvent *key_event = static_cast<const QKeyEvent*> (event);
+        const QKeyEvent *key_event = static_cast<const QKeyEvent*> (qevent);
         ScimBridgeKeyEvent *bridge_key_event = scim_bridge_key_event_qt_to_bridge (key_event);
         
         if (scim_bridge_key_event_get_code (bridge_key_event) == SCIM_BRIDGE_KEY_CODE_NullKey)
@@ -247,8 +370,13 @@ void ScimBridgeClientIMContextImpl::reset ()
     scim_bridge_pdebugln (5, "ScimBridgeClientIMContextImpl::reset ()");
 
     preedit_cursor_position = 0;
+#ifdef QT4
+    preedit_attributes.clear ();
+    preedit_attributes.push_back (QAttribute (QInputMethodEvent::Cursor, preedit_cursor_position, true, 0));
+#else
     preedit_selected_offset = 0;
     preedit_selected_length = 0;
+#endif
     preedit_string = "";
 
     if (scim_bridge_client_is_messenger_opened ()) {
@@ -257,7 +385,9 @@ void ScimBridgeClientIMContextImpl::reset ()
         }
     }
 
-    QInputContext::reset();
+#ifndef QT4
+    QInputContext::reset ();
+#endif
 }
 
 
@@ -282,7 +412,7 @@ void ScimBridgeClientIMContextImpl::focus_in ()
 {
     scim_bridge_pdebugln (8, "ScimBridgeClientIMContextImpl::focus_in ()");
 
-    if (focused_imcontext != NULL) focused_imcontext->focus_out ();
+    if (focused_imcontext != NULL && focused_imcontext != this) focused_imcontext->focus_out ();
 
     focused_imcontext = this;
 
@@ -332,12 +462,20 @@ void ScimBridgeClientIMContextImpl::commit ()
     scim_bridge_pdebugln (5, "ScimBridgeClientIMContextImpl::commit ()");
     
     if (commit_string.length () <= 0) return;
+#ifdef QT4
+    scim_bridge_pdebugln (9, "commit string: %s", commit_string.toUtf8 ().data ());
+#endif
 
     const bool is_composing = isComposing ();
 
+#ifdef QT4
+    QInputMethodEvent commit_event;
+    commit_event.setCommitString (commit_string);
+    sendEvent (commit_event);
+#else
     if (!is_composing) sendIMEvent (QEvent::IMStart);
-
     sendIMEvent (QEvent::IMEnd, commit_string);
+#endif
 
     if (is_composing) update_preedit ();
 }
@@ -352,7 +490,12 @@ void ScimBridgeClientIMContextImpl::forward_key_event (const ScimBridgeKeyEvent
         key_event_forwarded = true;
 #ifdef Q_WS_X11
         const WId window_id = focused_widget->winId ();
-        XEvent *x_event = scim_bridge_key_event_bridge_to_x11 (key_event, qt_xdisplay (), window_id);
+#ifdef QT4
+        Display *x11_display = QX11Info::display();
+#else
+        Display *x11_display = qt_xdisplay ();
+#endif
+        XEvent *x_event = scim_bridge_key_event_bridge_to_x11 (key_event, x11_display, window_id);
         qApp->x11ProcessEvent (x_event);
         free (x_event);
 #else
@@ -375,8 +518,13 @@ void ScimBridgeClientIMContextImpl::set_preedit_shown (bool shown)
     if (!preedit_shown) {
         preedit_string = "";
         preedit_cursor_position = 0;
+#ifdef QT4
+        preedit_attributes.clear ();
+        preedit_attributes.push_back (QAttribute (QInputMethodEvent::Cursor, preedit_cursor_position, true, 0));
+#else
         preedit_selected_offset = 0;
         preedit_selected_length = 0;
+#endif
     }
 }
 
@@ -399,10 +547,14 @@ void ScimBridgeClientIMContextImpl::set_preedit_attributes (ScimBridgeAttribute*
 {
     scim_bridge_pdebugln (5, "ScimBridgeClientIMContextImpl::set_preedit_attribute ()");
     
+#ifdef QT4
+    preedit_attributes.clear ();
+    preedit_attributes.push_back (QAttribute (QInputMethodEvent::Cursor, preedit_cursor_position, true, 0));
+#else
     preedit_selected_offset = 0;
     preedit_selected_length = 0;
+#endif
 
-    // Basically, preedit attributes are not supported in Qt-3.3.
     for (int i = 0; i < attribute_count; ++i) {
         const ScimBridgeAttribute *attribute = attributes[i];
         
@@ -411,11 +563,80 @@ void ScimBridgeClientIMContextImpl::set_preedit_attributes (ScimBridgeAttribute*
         const scim_bridge_attribute_type_t attribute_type = scim_bridge_attribute_get_type (attribute);
         const scim_bridge_attribute_value_t attribute_value = scim_bridge_attribute_get_value (attribute);
 
+#ifdef QT4
+        const size_t attribute_length = attribute_end - attribute_begin;
+
+        const QWidget *focused_widget = qApp->focusWidget ();
+        const QPalette &palette = focused_widget->palette ();
+
+        const QBrush &reversed_foreground = palette.base ();
+        const QBrush &reversed_background = palette.text ();
+        const QBrush &hilight_foreground = palette.highlightedText ();
+        const QBrush &hilight_background = palette.highlight ();
+
+        switch (attribute_type) {
+        case ATTRIBUTE_DECORATE:
+            switch (attribute_value) {
+            case SCIM_BRIDGE_ATTRIBUTE_DECORATE_HIGHLIGHT:
+                {
+                    QTextCharFormat text_format;
+                    text_format.setForeground (hilight_foreground);
+                    text_format.setBackground (hilight_background);
+                    QAttribute qt_attribute (QInputMethodEvent::TextFormat, attribute_begin, attribute_length, text_format);
+                    preedit_attributes.push_back (qt_attribute);
+                }
+                break;
+            case SCIM_BRIDGE_ATTRIBUTE_DECORATE_REVERSE:
+                {
+                    QTextCharFormat text_format;
+                    text_format.setForeground (reversed_foreground);
+                    text_format.setBackground (reversed_background);
+                    QAttribute qt_attribute (QInputMethodEvent::TextFormat, attribute_begin, attribute_length, text_format);
+                    preedit_attributes.push_back (qt_attribute);
+                }
+                break;
+            case SCIM_BRIDGE_ATTRIBUTE_DECORATE_UNDERLINE:
+                {
+                    QTextCharFormat text_format;
+                    text_format.setProperty (QTextFormat::FontUnderline, true);
+                    QAttribute qt_attribute (QInputMethodEvent::TextFormat, attribute_begin, attribute_length, text_format);
+                    preedit_attributes.push_back (qt_attribute);
+                }
+                break;
+            }
+            break;
+        case ATTRIBUTE_FOREGROUND:
+            {
+                QTextCharFormat text_format;
+                const unsigned int red = scim_bridge_attribute_get_red (attribute);
+                const unsigned int green = scim_bridge_attribute_get_green (attribute);
+                const unsigned int blue = scim_bridge_attribute_get_blue (attribute);
+                text_format.setForeground (QColor (red, green, blue));
+                QAttribute qt_attribute (QInputMethodEvent::TextFormat, attribute_begin, attribute_length, text_format);
+                preedit_attributes.push_back (qt_attribute);
+            }
+            break;
+        case ATTRIBUTE_BACKGROUND:
+            {
+                QTextCharFormat text_format;
+                const unsigned int red = scim_bridge_attribute_get_red (attribute);
+                const unsigned int green = scim_bridge_attribute_get_green (attribute);
+                const unsigned int blue = scim_bridge_attribute_get_blue (attribute);
+                text_format.setBackground (QColor (red, green, blue));
+                QAttribute qt_attribute (QInputMethodEvent::TextFormat, attribute_begin, attribute_length, text_format);
+                preedit_attributes.push_back (qt_attribute);
+            }
+            break;
+        default:
+            break;
+        }
+#else
         if (attribute_type == ATTRIBUTE_DECORATE && (attribute_value == SCIM_BRIDGE_ATTRIBUTE_DECORATE_HIGHLIGHT || attribute_value == SCIM_BRIDGE_ATTRIBUTE_DECORATE_REVERSE)) {
             preedit_selected_offset = attribute_begin;
             preedit_selected_length = attribute_end - attribute_begin;
             break;
         }
+#endif
     }
 }
 
@@ -424,9 +645,14 @@ void ScimBridgeClientIMContextImpl::update_preedit ()
 {
     scim_bridge_pdebugln (5, "ScimBridgeClientIMContextImpl::update_preedit ()");
 
+#ifdef QT4
+    preedit_attributes[0] = QAttribute (QInputMethodEvent::Cursor, preedit_cursor_position, true, 0);
+    QInputMethodEvent im_event (preedit_string, preedit_attributes);
+    sendEvent (im_event);
+    update ();
+#else
     if (preedit_shown) {
         if (!isComposing ()) sendIMEvent (QEvent::IMStart);
-
         const size_t preedit_length = preedit_string.length ();
 
         size_t cursor_position = preedit_cursor_position;
@@ -434,11 +660,11 @@ void ScimBridgeClientIMContextImpl::update_preedit ()
 
         size_t selected_length = preedit_selected_length;
         if (cursor_position + selected_length > preedit_length) selected_length = preedit_length - cursor_position;
-
         sendIMEvent (QEvent::IMCompose, preedit_string, cursor_position, selected_length);
     } else {
         if (isComposing ()) sendIMEvent (QEvent::IMEnd);
     }
+#endif
 }
 
 
@@ -468,7 +694,6 @@ void scim_bridge_client_imcontext_set_id (ScimBridgeClientIMContext *imcontext,
     imcontext_impl->set_id (new_id);
 }
 
-
 void scim_bridge_client_imcontext_beep (ScimBridgeClientIMContext *imcontext)
 {
     QApplication::beep ();
@@ -548,20 +773,42 @@ void scim_bridge_client_imcontext_update_preedit (ScimBridgeClientIMContext *imc
 
 boolean scim_bridge_client_imcontext_get_surrounding_text (ScimBridgeClientIMContext *imcontext, int before_max, int after_max, char **string, int *cursor_position)
 {
+#ifdef QT4
+    ScimBridgeClientIMContextImpl *imcontext_impl = static_cast<ScimBridgeClientIMContextImpl*> (imcontext);
+    return imcontext_impl->get_surrounding_text (before_max, after_max, string, cursor_position);
+#else
     scim_bridge_perrorln ("FIXME: scim_bridge_client_imcontext_get_surrounding_text () is not yet implemented.");
     return FALSE;
+#endif
 }
 
 
 boolean scim_bridge_client_imcontext_delete_surrounding_text (ScimBridgeClientIMContext *imcontext, int offset, int length)
 {
+#ifdef QT4
+    ScimBridgeClientIMContextImpl *imcontext_impl = static_cast<ScimBridgeClientIMContextImpl*> (imcontext);
+    return imcontext_impl->delete_surrounding_text (offset, length);
+#else
     scim_bridge_perrorln ("FIXME: scim_bridge_client_imcontext_delete_surrounding_text () is not yet implemented.");
     return FALSE;
+#endif
 }
 
 
 boolean scim_bridge_client_imcontext_replace_surrounding_text (ScimBridgeClientIMContext *imcontext, int cursor_position, const char *string)
 {
+#ifdef QT4
+    ScimBridgeClientIMContextImpl *imcontext_impl = static_cast<ScimBridgeClientIMContextImpl*> (imcontext);
+    return imcontext_impl->replace_surrounding_text (string, cursor_position);
+#else
     scim_bridge_perrorln ("FIXME: scim_bridge_client_imcontext_replace_surrounding_text () is not yet implemented.");
     return FALSE;
+#endif
 }
+
+
+void scim_bridge_client_imcontext_imengine_status_changed (ScimBridgeClientIMContext *imcontext, boolean enabled)
+{
+    // Do nothing.
+}
+
